# Cancer.r: Fit Univariate LZIP model to cancer genomics data
# Outcomes are y1=Endocytosis, y2=CCRinteraction, y3=MAPK       
# One latent factor
# Produces estimates given in Web Table 5
# Trace plots given in Web Figs 6,7
# Program takes about 10 min to run 125K iterations
# May 2016
#########################
rtpois <- function(n=n,mu=mu) qpois(runif(n, dpois(0, mu), 1), mu)  # Zero-truncated Poisson fn
set.seed(092415)

###############
# Import Data #
###############
data<-read.table("I:/Brian/LZIP/R/Final R Files/cancer.txt",header=T) # Change path as needed

####################
# Derive variables #
####################
 y1<-data[,1]             # Endocytosis
 y2<-data[,2]             # CCR-Interaction
 y3<-data[,3]             # MAPK
 n<-length(y1)            # Number of patients  
 stage1<-1*(data[,4]==1)  # Stage 1 
 stage3<-1*(data[,4]==3)  # Stage 3 (Note Stage 2 is "reference" group)
 x1<-stage1
 x3<-stage3
 X<-cbind(x1,x3)
  
##########
# Priors #
##########
 a<-b<-0.001                    # shape and rate for Gamma prior for lambdas, exp(beta1) and exp(beta2)

#########
# Inits #
#########
 beta11<-beta12<-beta21<-beta22<-beta31<-beta32<-c(0,0)   # Regression coefficients for stage 1 and stage 3
                                                          # for each outcome and model component (3 x 2 = 6)
 lambda1<-lambda2<-lambda3<-c(1,1)      # Factor loadings
 mu1<-mu2<-mu3<-matrix(1,n,2)           # mu_jk=lambda_jk*xi_i*exp(X_i%*%beta_jk)
 z1<-z2<-z3<-matrix(0,n,2)              # z_j[,2]= latent count among at-risk
 xi<-rep(1,n)                           # Latent factors

############
# Num Sims #
############
 nsim<-110000             # Number of MCMC Iterations
 thin<-25                 # Thinning interval
 burn<-10000	            # Burnin
 lastit<-(nsim-burn)/thin	# Last stored value

#########
# Store #
#########
  Beta11<-Beta12<-Beta21<-Beta22<-Beta31<-Beta32<-Lambda1<-Lambda2<-Lambda3<-matrix(0,lastit,2)

  starttime<-proc.time()
#########
# Gibbs #
#########
for (i in 1:nsim){
  
  # Update z1: Outcome 1, Binary Component
    z1[y1==0&z1[,2]>0,1]<-0
    z1[y1==0&z1[,2]==0,1]<-rpois(sum(1*(y1==0&z1[,2]==0)),mu1[y1==0&z1[,2]==0,1]) # No info, draw from Pois
    z1[y1>0,1]<-rtpois(sum(1*(y1>0)),mu1[y1>0,1])
  
  # Update z12: Outcome 1, Count Component
    z1[y1==0&z1[,1]==0,2]<-rpois(sum(1*(y1==0&z1[,1]==0)),mu1[y1==0&z1[,1]==0,2]) # No info, draw from Pois
    z1[z1[,1]>0,2]<-y1[z1[,1]>0]  
  
  # Update z21: Outcome 2, Binary Component
    z2[y2==0&z2[,2]>0,1]<-0
    z2[y2==0&z2[,2]==0,1]<-rpois(sum(1*(y2==0&z2[,2]==0)),mu2[y2==0&z2[,2]==0,1]) # No info, draw from Pois
    z2[y2>0,1]<-rtpois(sum(1*(y2>0)),mu2[y2>0,1])
  
  # Update z22: Outcome 2, Count Component
    z2[y2==0&z2[,1]==0,2]<-rpois(sum(1*(y2==0&z2[,1]==0)),mu2[y2==0&z2[,1]==0,2]) # No info, draw from Pois
    z2[z2[,1]>0,2]<-y2[z2[,1]>0]  
  
  # Update z31: Outcome 3, Binary Component
    z3[y3==0&z3[,2]>0,1]<-0
    z3[y3==0&z3[,2]==0,1]<-rpois(sum(1*(y3==0&z3[,2]==0)),mu3[y3==0&z3[,2]==0,1]) # No info, draw from Pois
    z3[y3>0,1]<-rtpois(sum(1*(y3>0)),mu3[y3>0,1])
  
  # Update z32: Outcome 3, Count Component
    z3[y3==0&z3[,1]==0,2]<-rpois(sum(1*(y3==0&z3[,1]==0)),mu3[y3==0&z3[,1]==0,2]) # No info, draw from Pois
    z3[z3[,1]>0,2]<-y3[z3[,1]>0]  
  
  
  # Update xi  NOTE: since only one latent factor (L=1), z_ij1=z_ij in eq. 13 of manuscript 
    shape<-1+apply(z1+z2+z3,1,sum)
    rate<-1+lambda1[1]*exp(X%*%beta11)+lambda1[2]*exp(X%*%beta12)+lambda2[1]*exp(X%*%beta21)+
          lambda2[2]*exp(X%*%beta22)+lambda3[1]*exp(X%*%beta31)+lambda3[2]*exp(X%*%beta32)
    xi<-rgamma(n,shape,rate)
  
  # Update lambda11
    shape<-a+sum(z1[,1])
    rate<-b+sum(xi*exp(X%*%beta11))
    lambda1[1]<-rgamma(1,shape,rate)
  
  # Update lambda12
    shape<-a+sum(z1[,2])
    rate<-b+sum(xi*exp(X%*%beta12))
    lambda1[2]<-rgamma(1,shape,rate)
  
  # Update lambda21
    shape<-a+sum(z2[,1])
    rate<-b+sum(xi*exp(X%*%beta21))
    lambda2[1]<-rgamma(1,shape,rate)
  
  # Update lambda22
    shape<-a+sum(z2[,2])
    rate<-b+sum(xi*exp(X%*%beta22))
    lambda2[2]<-rgamma(1,shape,rate)
  
  # Update lambda31
    shape<-a+sum(z3[,1])
    rate<-b+sum(xi*exp(X%*%beta31))
    lambda3[1]<-rgamma(1,shape,rate)
  
  # Update lambda32
    shape<-a+sum(z3[,2])
    rate<-b+sum(xi*exp(X%*%beta32))
    lambda3[2]<-rgamma(1,shape,rate)
  
  #################
  # Update beta11 #
  #################
  # beta111
    tmp<-rgamma(1,a+sum(z1[x1==1,1]),b+lambda1[1]*sum(xi[x1==1]*exp(x3[x1==1]*beta11[2]))) # Draw exp(beta111) from its Gamma full conditional
    beta11[1]<-log(tmp)
  
  # beta112
    tmp<-rgamma(1,a+sum(z1[x3==1,1]),b+lambda1[1]*sum(xi[x3==1]*exp(x1[x3==1]*beta11[1]))) # Draw exp(beta112) from its Gamma full conditional
    beta11[2]<-log(tmp)
  
    mu1[,1]<-lambda1[1]*xi*exp(X%*%beta11)
  
  #################
  # Update beta12 #
  #################  
  # beta121
    tmp<-rgamma(1,a+sum(z1[x1==1,2]),b+lambda1[2]*sum(xi[x1==1]*exp(x3[x1==1]*beta12[2])))
    beta12[1]<-log(tmp)
  
  # beta122
    tmp<-rgamma(1,a+sum(z1[x3==1,2]),b+lambda1[2]*sum(xi[x3==1]*exp(x1[x3==1]*beta12[1])))
    beta12[2]<-log(tmp)
  
    mu1[,2]<-lambda1[2]*xi*exp(X%*%beta12)
  
  #################
  # Update beta21 #
  ################# 
  # beta211
    tmp<-rgamma(1,a+sum(z2[x1==1,1]),b+lambda2[1]*sum(xi[x1==1]*exp(x3[x1==1]*beta21[2])))
    beta21[1]<-log(tmp)
  
  # beta212
    tmp<-rgamma(1,a+sum(z2[x3==1,1]),b+lambda2[1]*sum(xi[x3==1]*exp(x1[x3==1]*beta21[1])))
    beta21[2]<-log(tmp)
  
    mu2[,1]<-lambda2[1]*xi*exp(X%*%beta21)
  
  #################
  # Update beta22 #
  ################# 
  # beta221
    tmp<-rgamma(1,a+sum(z2[x1==1,2]),b+lambda2[2]*sum(xi[x1==1]*exp(x3[x1==1]*beta22[2])))
    beta22[1]<-log(tmp)
  
  # beta222
    tmp<-rgamma(1,a+sum(z2[x3==1,2]),b+lambda2[2]*sum(xi[x3==1]*exp(x1[x3==1]*beta22[1])))
    beta22[2]<-log(tmp)
  
    mu2[,2]<-lambda2[2]*xi*exp(X%*%beta22) 
  
  #################
  # Update beta31 #
  ################# 
  # beta311
    tmp<-rgamma(1,a+sum(z3[x1==1,1]),b+lambda3[1]*sum(xi[x1==1]*exp(x3[x1==1]*beta31[2])))
    beta31[1]<-log(tmp)
  
  # beta312
    tmp<-rgamma(1,a+sum(z3[x3==1,1]),b+lambda3[1]*sum(xi[x3==1]*exp(x1[x3==1]*beta31[1])))
    beta31[2]<-log(tmp)
  
    mu3[,1]<-lambda3[1]*xi*exp(X%*%beta31)
  
  #################
  # Update beta32 #
  #################  
  # beta321
    tmp<-rgamma(1,a+sum(z3[x1==1,2]),b+lambda3[2]*sum(xi[x1==1]*exp(x3[x1==1]*beta32[2])))
    beta32[1]<-log(tmp)
  
  # beta322
    tmp<-rgamma(1,a+sum(z3[x3==1,2]),b+lambda3[2]*sum(xi[x3==1]*exp(x1[x3==1]*beta32[1])))
    beta32[2]<-log(tmp)
  
    mu3[,2]<-lambda3[2]*xi*exp(X%*%beta32) 
  
  if (i> burn & i%%thin==0) {
    j<-(i-burn)/thin
    Lambda1[j,]<-lambda1	
    Lambda2[j,]<-lambda2
    Lambda3[j,]<-lambda3
    Beta11[j,]<-beta11
    Beta12[j,]<-beta12
    Beta21[j,]<-beta21
    Beta22[j,]<-beta22
    Beta31[j,]<-beta31
    Beta32[j,]<-beta32


  }
  
  if(i%%100==0) print(i)
}

runtime<-proc.time()-starttime

###########
# Results #
###########
 mlambda1<-colMeans(Lambda1)
 qlambda1<-apply(Lambda1,2,quantile,prob=c(0.025,0.975))
 mlambda2<-colMeans(Lambda2)
 qlambda2<-apply(Lambda2,2,quantile,prob=c(0.025,0.975))
 mlambda3<-colMeans(Lambda3)
 qlambda3<-apply(Lambda3,2,quantile,prob=c(0.025,0.975))

 mbeta11<-colMeans(Beta11)
 qbeta11<-apply(Beta11,2,quantile,prob=c(0.025,0.975))
 mbeta12<-colMeans(Beta12)
 qbeta12<-apply(Beta12,2,quantile,prob=c(0.025,0.975))
 mbeta21<-colMeans(Beta21)
 qbeta21<-apply(Beta21,2,quantile,prob=c(0.025,0.975))
 mbeta22<-colMeans(Beta22)
 qbeta22<-apply(Beta22,2,quantile,prob=c(0.025,0.975))
 mbeta31<-colMeans(Beta31)
 qbeta31<-apply(Beta31,2,quantile,prob=c(0.025,0.975))
 mbeta32<-colMeans(Beta32)
 qbeta32<-apply(Beta32,2,quantile,prob=c(0.025,0.975))

# Outcome 1
 print(mlambda1)
 print(qlambda1)
 print(mbeta11)
 print(qbeta11)
 print(mbeta12)
 print(qbeta12)
 
# Outcome 2
 print(mlambda2)
 print(qlambda2)
 print(mbeta21)
 print(qbeta21)
 print(mbeta22)
 print(qbeta22)

# Outcome 3
 print(mlambda3)
 print(qlambda3)
 print(mbeta31)
 print(qbeta31)
 print(mbeta32)
 print(qbeta32)

#####################   
# Trace Plots       #
# See Web Figs 6, 7 #
#####################

  #########
  # Betas #
  ######### 
  par(mfrow=c(4,3))
  # Beta11
    plot(1:lastit,Beta11[,1],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[111]),main=expression(beta[111]))
    abline(h=mbeta11[1],col="blue4")
    plot(1:lastit,Beta11[,2],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[112]),main=expression(beta[112]))
    abline(h=mbeta11[2],col="blue4")
  
  # Beta12
    plot(1:lastit,Beta12[,1],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[121]),main=expression(beta[121]))
    abline(h=mbeta12[1],col="blue4")
    plot(1:lastit,Beta12[,2],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[122]),main=expression(beta[122]))
    abline(h=mbeta12[2],col="blue4")

  # Beta21
    plot(1:lastit,Beta21[,1],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[211]),main=expression(beta[211]))
    abline(h=mbeta21[1],col="blue4")
    plot(1:lastit,Beta21[,2],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[212]),main=expression(beta[212]))
    abline(h=mbeta21[2],col="blue4")

  # Beta22
    plot(1:lastit,Beta22[,1],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[221]),main=expression(beta[221]))
    abline(h=mbeta22[1],col="blue4")
    plot(1:lastit,Beta22[,2],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[222]),main=expression(beta[222]))
    abline(h=mbeta22[2],col="blue4")

  # Beta31
    plot(1:lastit,Beta31[,1],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[311]),main=expression(beta[311]))
    abline(h=mbeta31[1],col="blue4")
    plot(1:lastit,Beta31[,2],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[312]),main=expression(beta[312]))
    abline(h=mbeta31[2],col="blue4")

  # Beta32
    plot(1:lastit,Beta32[,1],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[321]),main=expression(beta[321]))
    abline(h=mbeta32[1],col="blue4")
    plot(1:lastit,Beta32[,2],type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(beta[322]),main=expression(beta[322]))
    abline(h=mbeta32[2],col="blue4")

  ###########
  # Lambdas #
  ###########
  par(mfrow=c(3,2))

  # Lambda1
    plot(1:lastit,Lambda1[,1],type="l",col="lightgreen",xlab="Stored Iteration",
     ylab=expression(lambda[11]),main=expression(lambda[11]))
    abline(h=mlambda1[1],col="blue4")
    plot(1:lastit,Lambda1[,2],type="l",col="lightgreen",xlab="Stored Iteration",
     ylab=expression(lambda[12]),main=expression(lambda[12]))
    abline(h=mlambda1[2],col="blue4")

  # Lambda2
    plot(1:lastit,Lambda2[,1],type="l",col="lightgreen",xlab="Stored Iteration",
     ylab=expression(lambda[21]),main=expression(lambda[21]))
    abline(h=mlambda2[1],col="blue4")
    plot(1:lastit,Lambda2[,2],type="l",col="lightgreen",xlab="Stored Iteration",
     ylab=expression(lambda[22]),main=expression(lambda[22]))
    abline(h=mlambda2[2],col="blue4")

  # Lambda3
    plot(1:lastit,Lambda3[1:lastit,1],type="l",col="lightgreen",xlab="Stored Iteration",
     ylab=expression(lambda[31]),main=expression(lambda[31]))
    abline(h=mlambda3[1],col="blue4")
    plot(1:lastit,Lambda3[1:lastit,2],type="l",col="lightgreen",xlab="Stored Iteration",
     ylab=expression(lambda[32]),main=expression(lambda[32]))
    abline(h=mlambda3[2],col="blue4")
