# Sim1.r: R program for simulation 1, conventional ZIP
# Produces estimates from Table 1 corresponding to n=5000 and approx. 70% zeros 
# Trace Plots in Web Fig 1
# Note: For n=5000, takes appox. 45 sec to run 10K iterations, depending on machine
# May 2016
######################################
rtpois <- function(n=n,mu=mu) qpois(runif(n, dpois(0, mu), 1), mu)  # Zero-truncated Poisson fn

#################
# Generate Data #
#################
  set.seed(060115)
  n<-5000               # Sample size
  beta1<-c(-.5,-.5)     # Regression coeffs for binary component
  beta2<-c(1,-1)        # Regression coeffs for count component
  x<-rbinom(n,1,.5)     # Binary covariate
  X<-cbind(1,x)         # Design matrix
  n1<-sum(x)            # Sample size with x=1 (to estimate beta12 and beta22)
  mu1<-exp(X%*%beta1)
  mu2<-exp(X%*%beta2)
  phi<-1-exp(-mu1)      # cloglog link for "at-risk" probability, pr(y1=1)
  y1<-rbinom(n,1,phi)   # At-risk indicator
  y<-rep(0,n)           # Count outcome
  y[y1==0]<-0           # Structural zeros
  y[y1==1]<-rpois(sum(y1),mu2[y1==1])
  pzero<-table(y)[1]/n  # 68% zeros

##########
# Priors #
##########
  c<-d<-.001            # shape and rate for Gamma prior for exp(beta_21), exp(beta_22)

#########
# Inits #
#########
  beta1<-beta2<-c(0,0)
  mu1<-mu2<-rep(1,n)    # mu_k=exp(X%*%beta_k)

  z1<-z2<-rep(0,n)      # z1 = latent Poisson at-risk indicator, z2= count among at-risk

#########
# Store #
#########
  nsim<-10000
  Beta1<-Beta2<-matrix(0,nsim,2)

starttime<-proc.time()
#########
# Gibbs #
#########
for (i in 2:nsim){
 # Update z1 and parms
   z1[y==0&z2>0]<-0                                            # If z2 >0 and y=0, then z1 must = 0 to ensure y=0
   z1[y==0&z2==0]<-rpois(sum(1*(y==0&z2==0)),mu1[y==0&z2==0])  # No info, drax from Pois
   z1[y>0]<-rtpois(sum(1*(y>0)),mu1[y>0])                      # If y>0 then at-risk, so z1 must be >0 
   e1<-rgamma(1,c+sum(z1),d+sum(exp(x*beta1[2])))              # Draw exp(beta10) from gamma full conditional
   Beta1[i,1]<-beta1[1]<-log(e1)
   e2<-rgamma(1,c+sum(z1[x==1]),d+n1*e1)                       # exp(beta11)
   Beta1[i,2]<-beta1[2]<-log(e2)
   mu1<-exp(X%*%beta1)
  
 # Update z2 and parms
   z2[y==0&z1==0]<-rpois(sum(1*(y==0&z1==0)),mu2[y==0&z1==0])  # No info, drax from Pois
   z2[z1>0]<-y[z1>0]                                           # if z1>0, then at risk so z2=y
   e1<-rgamma(1,c+sum(z2),d+sum(exp(x*beta2[2])))              # exp(beta20)
   Beta2[i,1]<-beta2[1]<-log(e1)           
   e2<-rgamma(1,c+sum(z2[x==1]),d+n1*e1)                       # exp(beta21)
   Beta2[i,2]<-beta2[2]<-log(e2)
   mu2<-exp(X%*%beta2)
 
  if(i%%100==0) print(i)
}

runtime<-proc.time()-starttime

# Summary
  beta1<-colMeans(Beta1[5001:nsim,])                          # Post mean beta10,beta11 (binary component)
  q1<-apply(Beta1[5001:nsim,],2,quantile,prob=c(0.025,0.975)) # 95% Cred. Int. 
  beta2<-colMeans(Beta2[5001:nsim,])                          # Post mean beta20,beta21 (count component)
  q2<-apply(Beta2[5001:nsim,],2,quantile,prob=c(0.025,0.975)) # 95% Cred. Int.

###############################
# Print Results --See Table 1 #
###############################
  print(beta1)
  print(q1)
  print(beta2)
  print(q2)

###################
# Trace Plots     #
# See Web Fig 1   #
###################
  par(mfrow=c(2,2))
  plot(1:nsim,Beta1[,1],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[10]),main=expression(beta[10]))
  abline(h=beta1[1],col="blue4")
  plot(1:nsim,Beta1[,2],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[11]),main=expression(beta[11]))
  abline(h=beta1[2],col="blue4")

  plot(1:nsim,Beta2[,1],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[20]),main=expression(beta[20]))
  abline(h=beta2[1],col="blue4")
  plot(1:nsim,Beta2[,2],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[21]),main=expression(beta[21]))
  abline(h=beta2[2],col="blue4")

