# Sim2.r
# R program for simulation 2, One-Factor Univariate LZIP
# Produces estimates from Table 2 corresponding to n=500 and ~70% zeros 
# See Table 2, rows 9-12 and Web Figure 2 for Trace Plots
# Note: For n=500, takes about 6 sec to run 10K iterations, depending on machine
# May 2016
###########################################
rtpois <- function(n=n,mu=mu) qpois(runif(n, dpois(0, mu), 1), mu)  # Zero-truncated Poisson fn

#################
# Generate Data #
#################
  set.seed(081415)
  n<-500
  beta1<--.5            # Coeff for x, binary component 
  beta2<-.75            # Coeff for x, count component
  x<-rbinom(n,1,.5)     # Covariate
  n1<-sum(x)            # Sample size with x=1 (to estimate beta1 and beta2)
  lambda1<-.5           # Loading for zero part
  lambda2<-1.5          # Loading for count part
  xi<-rgamma(n,1,1)     # Latent factor
  mu1<-lambda1*xi*exp(x*beta1)  # mean of z1, where z1 is latent Poisson for binary component
  mu2<-lambda2*xi*exp(x*beta2)  # mean of z2, latent Poisson for count component

  phi<-1-exp(-mu1)      # cloglog link for "at-risk" probability, pr(y1=1)
  y1<-rbinom(n,1,phi)   # At-risk indicator -- change to 1-phi to fit zeroinfl function below
  y<-rep(0,n)           # Count outcome
  y[y1==0]<-0           # Structural zeros
  y[y1==1]<-rpois(sum(y1),mu2[y1==1]) 
  pzero<-table(y)[1]/n  #  77% zeros
  
##########
# Priors #
##########
  a<-b<-0.001           # shape and rate for Gamma prior for lambdas, exp(beta1) and exp(beta2)

#########
# Inits #
#########
  beta<-c(0,0)
  lambda1<-lambda2<-1   # Factor loadings for binary and count components
  mu1<-mu2<-rep(1,n)    # mu_k=lambda_k*xi_i*exp(x_i%*%beta_k)
  z1<-z2<-rep(0,n)      # z2= latent count among at-risk
  xi<-rep(1,n)

#########
# Store #
#########
  nsim<-10000
  Beta<-Lambda<-matrix(0,nsim,2)
  Lambda[1,]<-c(1,1)

starttime<-proc.time()
#########
# Gibbs #
#########
for (i in 2:nsim){
  
  # Update z1 
  z1[y==0&z2>0]<-0
  z1[y==0&z2==0]<-rpois(sum(1*(y==0&z2==0)),mu1[y==0&z2==0]) # No info, draw from Pois
  z1[y>0]<-rtpois(sum(1*(y>0)),mu1[y>0])
  
  # Update z2
  z2[y==0&z1==0]<-rpois(sum(1*(y==0&z1==0)),mu2[y==0&z1==0]) # No info, draw from Pois
  z2[z1>0]<-y[z1>0]  
  
  # Update xi  NOTE: since only one latent factor (L=1), s_ij1=z_ij
  shape<-1+z1+z2
  rate<-1+lambda1*exp(x*beta[1])+lambda2*exp(x*beta[2])
  xi<-rgamma(n,shape,rate)
  
  # Update lambda1 = factor laoding for binary component
  shape<-a+sum(z1)
  rate<-b+sum(xi*exp(x*beta[1]))
  Lambda[i,1]<-lambda1<-rgamma(1,shape,rate)
  
  # Update lambda2 = factor loading for count component
  shape<-a+sum(z2)
  rate<-b+sum(xi*exp(x*beta[2]))
  Lambda[i,2]<-lambda2<-rgamma(1,shape,rate)
  
  # Update beta1
  tmp<-rgamma(1,a+sum(z1[x==1]),b+lambda1*sum(xi[x==1]))  # draw exp(beta1) from gamma full conditional
  Beta[i,1]<-beta[1]<-log(tmp)
  mu1<-lambda1*xi*exp(x*beta[1])
  
 # Update beta2
  tmp<-rgamma(1,a+sum(z2[x==1]),b+lambda2*sum(xi[x==1]))  # exp(beta2)
  Beta[i,2]<-beta[2]<-log(tmp)
  mu2<-lambda2*xi*exp(x*beta[2])
 
  if(i%%100==0) print(i)
}

runtime<-proc.time()-starttime

#########################
# Results: See Table 2  #
# Rows 9-16 in Paper    #
#########################
 mlambda<-colMeans(Lambda[5001:nsim,])
 qlambda<-apply(Lambda[5001:nsim,],2,quantile,prob=c(0.025,0.975))
 mbeta<-colMeans(Beta[5001:nsim,])
 qbeta<-apply(Beta[5001:nsim,],2,quantile,prob=c(0.025,0.975))
 print(mlambda)   # lambda_1 (binary component) and lambda_2 (count component)
 print(qlambda)
 print(mbeta)     # beta_1 (binary component) and beta_2 (count component)
 print(qbeta)

#################
# Trace Plots   #
# See Web Fig 2 #
#################
  par(mfrow=c(2,2))

  plot(1:nsim,Lambda[1:nsim,1],type="l",col="lightgreen",xlab="Iteration",
       ylab=expression(lambda[1]),main=expression(lambda[1]))
  abline(h=mlambda[1],col="blue4")
  plot(1:nsim,Lambda[1:nsim,2],type="l",col="lightgreen",xlab="Iteration",
       ylab=expression(lambda[2]),main=expression(lambda[2]))
  abline(h=mlambda[2],col="blue4")

  plot(1:nsim,Beta[1:nsim,1],type="l",col="lightgreen",xlab="Iteration",
       ylab=expression(beta[1]),main=expression(beta[1]))
  abline(h=mbeta[1],col="blue4")
  plot(1:nsim,Beta[1:nsim,2],type="l",col="lightgreen",xlab="Iteration",
       ylab=expression(beta[2]),main=expression(beta[2]))
  abline(h=mbeta[2],col="blue4")

