# Sim3.r: Results for Sim 3
# 2 outcomes with 2 latent factors linking structural zero and at-risk count
# n=500, nsim=125K, ~40% zeros
# 2 Covariates: 1 binary and 1 continuous (use MH)
# Produces estimates from Table 3, Rows 1-16
# Trace Plots are presented in Web Figs 4 and 5
# NOTE: For n=500, takes about 14 min to run 125K sims, depending on machine
# May 2016
################################################
rtpois <- function(n=n,mu=mu) qpois(runif(n, dpois(0, mu), 1), mu)  # Zero-truncated Poisson fn

#################
# Generate Data #
#################
  set.seed(091515)
  n<-500
  beta11<-c(1,.5)                     # Regression coefs. for outcome 1, binary
  beta12<-c(.25,-.25)                 # Regression coefs. for outcome 1, count
  beta21<-c(.75,.25)                  # Regression coefs. for outcome 2, binary
  beta22<-c(.5,-.5)                   # Regression coefs. for outcome 2, count
  x1<-rbinom(n,1,.75)                 # Subject-specific binary covariate 
  x2<-rnorm(n,0,sd=2)                 # Subject-specific continuous outcome
  X<-cbind(x1,x2)                     # Design matrix (Note: no fixed intercept)
  n1<-sum(x1)                         # Sample size with x=1 (to estimate betas)

  lambda11<-c(2.5,0)                  # Loadings for outcome 1, binary component
  lambda12<-c(0,2.5)                  # Loadings for outcome 1, count component
  lambda21<-c(2.5,0)                  # Loadings for outcome 2, binary component
  lambda22<-c(0,2.5)                  # Loadings for outcome 2, count component

  xi<-matrix(rgamma(2*n,1,1),n,2)     # Subject-specific latent factors
  mu11<-xi%*%lambda11*exp(X%*%beta11) # Latent Poisson mean for outcome 1, binary part
  mu12<-xi%*%lambda12*exp(X%*%beta12) # Latent Poisson mean for outcome 1, count part
  mu21<-xi%*%lambda21*exp(X%*%beta21) # Latent Poisson mean for outcome 2, binary part
  mu22<-xi%*%lambda22*exp(X%*%beta22) # Latent Poisson mean for outcome 2, count part

# Outcome 1
  phi<-1-exp(-mu11)      # cloglog link for "at-risk" probability, pr(y1=1) for outcome 1
  ytmp<-rbinom(n,1,phi)  # At-risk indicator -- change to 1-phi to fit zeroinfl function below
  y1<-rep(0,n)           # Outcome 1
  y1[ytmp==0]<-0         # Structural zeros
  y1[ytmp==1]<-rpois(sum(ytmp),mu12[ytmp==1]) # For at risk class, sample from Poisson with mean mu12
  pzero1<-table(y1)[1]/n # 41% zeros


# Outcome 2
  phi<-1-exp(-mu21)       # cloglog link for "at-risk" probability, pr(y1=1) for outcome 2
  ytmp<-rbinom(n,1,phi)   # At-risk indicator
  y2<-rep(0,n)            # Outcome 2
  y2[ytmp==0]<-0          # Structural zeros
  y2[ytmp==1]<-rpois(sum(ytmp),mu22[ytmp==1]) # For at risk class, sample from Poisson with mean mu22
  pzero2<-table(y2)[1]/n  # 41 % zeros

##########
# Priors #
##########
  a<-b<-.001              # shape and rate for Gamma prior for lambdas, exp(beta_11), exp(beta_12), etc.

######### 
# Inits #
#########
  beta11<-beta12<-beta21<-beta22<-c(0,0)         # Regression coeff's for outcomes 1 and 2 (binary and count)
  # If needed, can start lambdas at true values to help ensure correct labeling, but this is not necessary
  lambda11<-lambda12<-lambda21<-lambda22<-c(1,1)  # Factor loadings for outcome j, component k (factors 1 and 2)
  z1<-z2<-matrix(0,n,2)                           # z_j[,k]   = latent count for outcome j, component k 
  mu1<-mu2<-matrix(1,n,2)                         # mu_j[,k]  = mean of z_j[,k]
  s11<-s12<-s21<-s22<-matrix(1,n,2)               # s_jk[,l]  = Latent Poisson RVs for outcome j, component k, factors 1,2
                                                  #             IMPORTANT NOTE: these are denoted z_ijkl in paper -- see eq. 13 
                                                  # Note: zj[,k]=s_jk[,1]+s_jk[,2]=apply(s_jk,1,sum)
  mu11<-mu12<-mu21<-mu22<-matrix(.5,n,2)          # mu_jk[,l] = mean of sjkl, for outcome j, component k, factor l=1,2
                                                  # Note: mu_j[,k]=mu_jk[,1]+mu_jk[,2]=apply(mu_jk,1,sum)
############
# Num Sims #
############
  nsim<-125000              # Number of MCMC Iterations
  thin<-25				          # Thinning interval
  burn<-25000	              # Burnin
  lastit<-(nsim-burn)/thin	# Last stored value

#########
# Store #
#########
  Beta11<-Beta12<-Beta21<-Beta22<-Lambda11<-Lambda12<-Lambda21<-Lambda22<-matrix(0,lastit,2)
  Lambda11[1,]<-Lambda12[1,]<-Lambda21[1,]<-Lambda22[1,]<-c(1,1)
  A1<-A2<-A3<-A4<-0         # Acceptance Indicators for MH steps

starttime<-proc.time()
#########
# GIBBS #
#########
for (i in 2:nsim){
  
  ##############
  # Update z's #
  ##############
  # Update z1: Outcome 1, Binary Component
    z1[y1==0&z1[,2]>0,1]<-0
    z1[y1==0&z1[,2]==0,1]<-rpois(sum(1*(y1==0&z1[,2]==0)),mu1[y1==0&z1[,2]==0,1]) # No info, draw from Pois
    z1[y1>0,1]<-rtpois(sum(1*(y1>0)),mu1[y1>0,1])
  
  # Update z12: Outcome 1, Count Component
    z1[y1==0&z1[,1]==0,2]<-rpois(sum(1*(y1==0&z1[,1]==0)),mu1[y1==0&z1[,1]==0,2]) # No info, draw from Pois
    z1[z1[,1]>0,2]<-y1[z1[,1]>0]  
  
  # Update z21: Outcome 2, Binary Component
    z2[y2==0&z2[,2]>0,1]<-0
    z2[y2==0&z2[,2]==0,1]<-rpois(sum(1*(y2==0&z2[,2]==0)),mu2[y2==0&z2[,2]==0,1]) # No info, draw from Pois
    z2[y2>0,1]<-rtpois(sum(1*(y2>0)),mu2[y2>0,1])
  
  # Update z22: Outcome 2, Count Component
    z2[y2==0&z2[,1]==0,2]<-rpois(sum(1*(y2==0&z2[,1]==0)),mu2[y2==0&z2[,1]==0,2]) # No info, draw from Pois
    z2[z2[,1]>0,2]<-y2[z2[,1]>0]  
  
  ###############################################################
  # Update sikl's from multinomial full conditional             #
  # Note: These are denoted by zikl in paper (e.g., see eq 13)  #
  ###############################################################
  # Update s11: Outcome 1, binary component, latent factors 1,2
    pi<-mu11[,1]/mu1[,1]          # mu1[,1]=mu11[,1]+mu11[,2] = apply(mu11,1,sum) -- See update for betas
    s11[,1]<-rbinom(n,z1[,1],pi)  # In paper, s11 is z11
    s11[,2]<-z1[,1]-s11[,1]
  
  # Update s12: Outcome 1, count component, latent factors 1,2
    pi<-mu12[,1]/mu1[,2]
    s12[,1]<-rbinom(n,z1[,2],pi)
    s12[,2]<-z1[,2]-s12[,1]
  
  # Update s21: Outcome 2, binary component, latent factors 1,2
    pi<-mu21[,1]/mu2[,1]
    s21[,1]<-rbinom(n,z2[,1],pi)
    s21[,2]<-z2[,1]-s21[,1]
  
  # Update s22: Outcome 2, count component, latent factors 1,2
    pi<-mu22[,1]/mu2[,2]
    s22[,1]<-rbinom(n,z2[,2],pi)
    s22[,2]<-z2[,2]-s22[,1]
  
  ##############
  # Update xis #
  ##############
  # Update xi1  
    shape<-1+s11[,1]+s12[,1]+s21[,1]+s22[,1]
    rate<-1+lambda11[1]*exp(X%*%beta11)+lambda12[1]*exp(X%*%beta12)+lambda21[1]*exp(X%*%beta21)+lambda22[1]*exp(X%*%beta22)
    xi[,1]<-rgamma(n,shape,rate)
  
  # Update xi2  
    shape<-1+s11[,2]+s12[,2]+s21[,2]+s22[,2]
    rate<-1+lambda11[2]*exp(X%*%beta11)+lambda12[2]*exp(X%*%beta12)+lambda21[2]*exp(X%*%beta21)+lambda22[2]*exp(X%*%beta22) 
    xi[,2]<-rgamma(n,shape,rate)
  
  ################## 
  # Update lambdas #
  ##################
  # Outcome 1
  # lambda111 = Outcome 1, binary component, loading 1
    shape<-a+sum(s11[,1])
    rate<-b+sum(xi[,1]*exp(X%*%beta11))
    lambda11[1]<-rgamma(1,shape,rate)
  
  # lambda112 = Outcome 1, binary component, loading 2
    shape<-a+sum(s11[,2])
    rate<-b+sum(xi[,2]*exp(X%*%beta11))
    lambda11[2]<-rgamma(1,shape,rate)
  
  # lambda121 = Outcome 1, count component, loading 1
    shape<-a+sum(s12[,1])
    rate<-b+sum(xi[,1]*exp(X%*%beta12))
    lambda12[1]<-rgamma(1,shape,rate)
  
  # lambda122 = Outcome 1, count component, loading 2
    shape<-a+sum(s12[,2])
    rate<-b+sum(xi[,2]*exp(X%*%beta12))
    lambda12[2]<-rgamma(1,shape,rate)

  # Outcome 2
  # lambda211 = Outcome 2, binary component, loading 1
    shape<-a+sum(s21[,1])
    rate<-b+sum(xi[,1]*exp(X%*%beta21))
    lambda21[1]<-rgamma(1,shape,rate)
  
  # lambda212 = Outcome 2, binary component, loading 2
    shape<-a+sum(s21[,2])
    rate<-b+sum(xi[,2]*exp(X%*%beta21))
    lambda21[2]<-rgamma(1,shape,rate)
  
  # lambda221 = Outcome 2, count component, loading 1
    shape<-a+sum(s22[,1])
    rate<-b+sum(xi[,1]*exp(X%*%beta22))
    lambda22[1]<-rgamma(1,shape,rate)
  
  # lambda222 = Outcome 2, count component, loading 2
    shape<-a+sum(s22[,2])
    rate<-b+sum(xi[,2]*exp(X%*%beta22))
    lambda22[2]<-rgamma(1,shape,rate)
  
  ################
  # Update betas #
  ################
  ####################
  # Beta11 and mu11  #
  ####################
  # Update beta111: Outcome 1, binary part, covariate x1
    tmp<-rgamma(1,a+sum(z1[x1==1,1]),b+sum(xi[x1==1,]%*%lambda11*exp(x2[x1==1]*beta11[2])))
    beta11[1]<-log(tmp)
  
  # Update beta112: Outcome 1, binary part, covariate x2: USE MH  
    curmu11<-xi%*%lambda11*exp(X%*%beta11)    # Current mu11 
    l<-sum(dpois(z1[,1],curmu11,log=T))       # Current Log-Likelihood for z11 (outcome 1, binary)
  
    bstar<-beta11[2]+0.01*rt(1,3)            # Candidate beta112
    b11<-c(beta11[1],bstar)            
    mustar<-xi%*%lambda11*exp(X%*%b11)   
    lstar<-sum(dpois(z1[,1],mustar,log=T))    # Log-likelihood for z11 with candidate beta112
  
    r<-lstar+dnorm(bstar,0,sqrt(1000),log=T)-  
       (l+dnorm(beta11[2],0,sqrt(1000),log=T))# Acceptance Ratio
  
    if(log(runif(1))<r) {
      beta11[2]<-bstar
      A1<-A1+1
    }
  
  # Update mu11 = E(z11)
    mu11[,1]<-lambda11[1]*xi[,1]*exp(X%*%beta11)  # Mean of s111
    mu11[,2]<-lambda11[2]*xi[,2]*exp(X%*%beta11)  # Mean of s112
    mu1[,1]<-apply(mu11,1,sum)                    # Mean of z11
  
  ####################
  # Beta12 and mu12  #
  ####################
  # Update beta121: Outcome 1, count part, covariate x1
    tmp<-rgamma(1,a+sum(z1[x1==1,2]),b+sum(xi[x1==1,]%*%lambda12*exp(x2[x1==1]*beta12[2]))) 
    beta12[1]<-log(tmp)
  
  # Update beta122: Outcome 1, count part, covariate x2: USE MH  
    curmu12<-xi%*%lambda12*exp(X%*%beta12)    # Current mu12 
    l<-sum(dpois(z1[,2],curmu12,log=T))       # Current Log-Likelihood for z12 (outcome 1, count)
  
    bstar<-beta12[2]+0.025*rt(1,3)            # Candidate beta122
    b12<-c(beta12[1],bstar)            
    mustar<-xi%*%lambda12*exp(X%*%b12)   
    lstar<-sum(dpois(z1[,2],mustar,log=T))    # Log-likelihood for z12 with candidate beta122
  
    r<-lstar+dnorm(bstar,0,sqrt(1000),log=T)-  
       (l+dnorm(beta12[2],0,sqrt(1000),log=T))# Acceptance Ratio
  
    if(log(runif(1))<r) {
      beta12[2]<-bstar
      A2<-A2+1
    }
  
  # Update mu12 = E(z12)
    mu12[,1]<-lambda12[1]*xi[,1]*exp(X%*%beta12)  # Mean of s121
    mu12[,2]<-lambda12[2]*xi[,2]*exp(X%*%beta12)  # Mean of s122
    mu1[,2]<-apply(mu12,1,sum)                    # Mean of z12
    
  ####################
  # Beta21 and mu21  #
  ####################
  # Update beta211: Outcome 2, binary part, covariate x1
    tmp<-rgamma(1,a+sum(z2[x1==1,1]),b+sum(xi[x1==1,]%*%lambda21*exp(x2[x1==1]*beta21[2]))) 
    beta21[1]<-log(tmp)
  
  # Update beta212: Outcome 2, binary part, covariate x2: USE MH  
    curmu21<-xi%*%lambda21*exp(X%*%beta21)    # Current mu21 
    l<-sum(dpois(z2[,1],curmu21,log=T))       # Current Log-Likelihood for z21 (outcome 2, binary)
  
    bstar<-beta21[2]+0.025*rt(1,3)            # Candidate beta212
    b21<-c(beta21[1],bstar)            
    mustar<-xi%*%lambda21*exp(X%*%b21)   
    lstar<-sum(dpois(z2[,1],mustar,log=T))    # Log-likelihood for z21 with candidate beta212
  
    r<-lstar+dnorm(bstar,0,sqrt(1000),log=T)-  
       (l+dnorm(beta21[2],0,sqrt(1000),log=T))# Acceptance Ratio
  
    if(log(runif(1))<r) {
      beta21[2]<-bstar
      A3<-A3+1
    }
  
  # Update mu21 = E(z21)
    mu21[,1]<-lambda21[1]*xi[,1]*exp(X%*%beta21)  # Mean of s211
    mu21[,2]<-lambda21[2]*xi[,2]*exp(X%*%beta21)  # Mean of s212
    mu2[,1]<-apply(mu21,1,sum)                    # Mean of z21

  ####################
  # Beta22 and mu22  #
  ####################
  # Update beta221: Outcome 2, count part, covariate x1
    tmp<-rgamma(1,a+sum(z2[x1==1,2]),b+sum(xi[x1==1,]%*%lambda22*exp(x2[x1==1]*beta22[2]))) 
    beta22[1]<-log(tmp)
  
  # Update beta112: Outcome 2, count part, covariate x2: USE MH  
    curmu22<-xi%*%lambda22*exp(X%*%beta22)    # Current mu22 
    l<-sum(dpois(z2[,2],curmu22,log=T))       # Current Log-Likelihood for z22 (outcome 2, count)
  
    bstar<-beta22[2]+0.01*rt(1,3)            # Candidate beta112
    b22<-c(beta22[1],bstar)            
    mustar<-xi%*%lambda22*exp(X%*%b22)   
    lstar<-sum(dpois(z2[,2],mustar,log=T))    # Log-likelihood for z22 with candidate beta222
  
    r<-lstar+dnorm(bstar,0,sqrt(1000),log=T)-  
       (l+dnorm(beta22[2],0,sqrt(1000),log=T))# Acceptance Ratio
  
    if(log(runif(1))<r) {
      beta22[2]<-bstar
      A4<-A4+1
    }
  
  # Update mu22 = E(z22)
    mu22[,1]<-lambda22[1]*xi[,1]*exp(X%*%beta22)  # Mean of s221
    mu22[,2]<-lambda22[2]*xi[,2]*exp(X%*%beta22)  # Mean of s222
    mu2[,2]<-apply(mu22,1,sum)                    # Mean of z22
    
  #################
  # Store Results #
  #################
    if (i> burn & i%%thin==0) {
      j<-(i-burn)/thin
      Lambda11[j,]<-lambda11      # lambda_111 and lambda_112
      Lambda12[j,]<-lambda12      # lambda_121 and lambda_122
      Lambda21[j,]<-lambda21      # lambda_211 and lambda_212  
      Lambda22[j,]<-lambda22      # lambda_221 and lambda_222
      Beta11[j,]<-beta11          # beta_111 and beta_112
      Beta12[j,]<-beta12          # beta_121 and beta_122
      Beta21[j,]<-beta21          # beta_211 and beta_212
      Beta22[j,]<-beta22          # beta_221 and beta_222
    }
      
  if(i%%100==0) print(i)

}

runtime<-proc.time()-starttime

#########################
# Posterior Summaries   #
# See Table 3 Rows 1-16 #
#########################
  mbeta11<-colMeans(Beta11)  # Mean of beta111, beta1112
  mbeta12<-colMeans(Beta12)
  mbeta21<-colMeans(Beta21)
  mbeta22<-colMeans(Beta22)
  qbeta11<-apply(Beta11,2,quantile,prob=c(0.025,0.975))  # CI for beta11
  qbeta12<-apply(Beta12,2,quantile,prob=c(0.025,0.975))
  qbeta21<-apply(Beta21,2,quantile,prob=c(0.025,0.975))
  qbeta22<-apply(Beta22,2,quantile,prob=c(0.025,0.975))

  mlambda11<-colMeans(Lambda11)
  qlambda11<-apply(Lambda11,2,quantile,prob=c(0.025,0.975))
  mlambda12<-colMeans(Lambda12)
  qlambda12<-apply(Lambda12,2,quantile,prob=c(0.025,0.975))
  mlambda21<-colMeans(Lambda21)
  qlambda21<-apply(Lambda21,2,quantile,prob=c(0.025,0.975))
  mlambda22<-colMeans(Lambda22)
  qlambda22<-apply(Lambda22,2,quantile,prob=c(0.025,0.975))

# Lambdas 
# NOTE 1: Lambda11 is 2x1 vector containing lambda_111 and lambda_112 in Table 3
#         Lambda12 is 2x1 vector containing lambda_121 and lambda_122, and so on
# NOTE 2: lambda111 and lambda112, etc. "switched" from simulated data, so need to account for this
  cat("lambda_111","\n",mlambda11[2],"(",qlambda11[,2],")","\n",
      "lambda_112","\n",mlambda11[1], "(",qlambda11[,1],")","\n",
      "lambda_121","\n",mlambda12[2],"(",qlambda12[,2],")","\n", 
      "lambda_122","\n",mlambda12[1], "(",qlambda12[,1],")","\n",
      "lambda_211","\n",mlambda21[2],"(",qlambda21[,2],")","\n", 
      "lambda_212","\n",mlambda21[1], "(",qlambda21[,1],")","\n",
      "lambda_221","\n",mlambda22[2],"(",qlambda22[,2],")","\n",
      "lambda_222","\n",mlambda22[1], "(",qlambda22[,1],")") 
  
  # Betas
  print("Betas")
  print(mbeta11)    # beta_111 and beta_112 = Outcome 1, binary component
  print(qbeta11)
  print(mbeta12)    # beta_121 and beta_122 = Outcome 1, count component
  print(qbeta12)
  print(mbeta21)    # beta_211 and beta_212 = Outcome 2, binary component
  print(qbeta21)
  print(mbeta22)    # beta_221 and beta_222 = Outcome 2, count component
  print(qbeta22)
  
#########################
# Trace Plots           #
# See Web Figs 4 and 5  #
#########################
  # Betas
  par(mfrow=c(4,2))
  # Beta 11
    plot(1:lastit,Beta11[,1],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(beta[111]),main=expression(beta[111]))
    abline(h=mbeta11[1],col="blue4")
    plot(1:lastit,Beta11[,2],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(beta[112]),main=expression(beta[112]))
    abline(h=mbeta11[2],col="blue4")

  # Beta 12
    plot(1:lastit,Beta12[,1],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(beta[121]),main=expression(beta[121]))
    abline(h=mbeta12[1],col="blue4")
    plot(1:lastit,Beta12[,2],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(beta[122]),main=expression(beta[122]))
    abline(h=mbeta12[2],col="blue4")

  # Beta 21
    plot(1:lastit,Beta21[,1],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(beta[211]),main=expression(beta[211]))
    abline(h=mbeta21[1],col="blue4")
    plot(1:lastit,Beta21[,2],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(beta[212]),main=expression(beta[212]))
    abline(h=mbeta21[2],col="blue4")

  # Beta 22
    plot(1:lastit,Beta22[,1],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(beta[221]),main=expression(beta[221]))
    abline(h=mbeta22[1],col="blue4")
    plot(1:lastit,Beta22[,2],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(beta[222]),main=expression(beta[222]))
    abline(h=mbeta22[2],col="blue4")
  

  ###########
  # Lambdas #
  ###########
    par(mfrow=c(4,2))
  # NOTE: ORDER OF FACTORS IS REVERSE, SO NEED TO ACCOUNT FOR THIS 
  # lambda 111
    plot(1:lastit,Lambda11[,2],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(lambda[111]),main=expression(lambda[111]))
    abline(h=mlambda11[2],col="blue4")

  # lambda 112
    plot(1:lastit,Lambda11[,1],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(lambda[112]),main=expression(lambda[112]))
    abline(h=mlambda11[1],col="blue4")

  # lambda 121
    plot(1:lastit,Lambda12[,2],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(lambda[121]),main=expression(lambda[121]))
    abline(h=mlambda12[2],col="blue4")

  # lambda 122
    plot(1:lastit,Lambda12[,1],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(lambda[122]),main=expression(lambda[122]))
    abline(h=mlambda12[1],col="blue4")

  # lambda 211
    plot(1:lastit,Lambda21[,2],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(lambda[211]),main=expression(lambda[211]))
    abline(h=mlambda21[2],col="blue4")

  # lambda 212
    plot(1:lastit,Lambda21[,1],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(lambda[212]),main=expression(lambda[212]))
    abline(h=mlambda21[1],col="blue4")

  # lambda 221
    plot(1:lastit,Lambda22[,2],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(lambda[221]),main=expression(lambda[221]))
    abline(h=mlambda22[2],col="blue4")

  # lambda 222
    plot(1:lastit,Lambda22[,1],type="l",col="lightgreen",xlab="Stored Iteration",
         ylab=expression(lambda[222]),main=expression(lambda[222]))
    abline(h=mlambda22[1],col="blue4")

